﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using Telerik.Charting;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ChartView;

namespace WpfApplication60
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();           

            this.DataContext = this.GetData();
            this.chart.AddHandler(RadCartesianChart3D.MouseLeftButtonDownEvent, new MouseButtonEventHandler(Chart_MouseLeftButtonDown), true);
        }

        private ObservableCollection<PlotInfo> GetData()
        {
            var result = new ObservableCollection<PlotInfo>()
            {
                new PlotInfo() { XValue = "A", YValue = "A", ZValue = 23 },
                new PlotInfo() { XValue = "B", YValue = "B", ZValue = 33 },
                new PlotInfo() { XValue = "C", YValue = "C", ZValue = 55 },
                new PlotInfo() { XValue = "D", YValue = "D", ZValue = 11 },
            };
            return result;
        }

        private void Chart_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Viewport3D viewport = ChildrenOfTypeExtensions.FindChildByType<Viewport3D>(this.chart);
            Point position = e.GetPosition(this.chart);
            Visual3D visualHit = HitTest(viewport, position);
            
            if (visualHit != null)
            {
                RadRect3D radLayoutSlot = GetLayoutSlot(visualHit);

                foreach (BarSeries3D series in this.chart.Series)
                {
                    foreach (XyzDataPoint3D dp in series.DataPoints)
                    {
                        if (dp.LayoutSlot.Equals(radLayoutSlot))
                        {
                            this.UpdateSelection((PlotInfo)dp.DataItem);
                            this.ResetMaterialSelector(series);
                            return;
                        }
                    }
                }
            }
        }

        private void UpdateSelection(PlotInfo plotInfo, bool isSingleSelection = true)
        {
            if (isSingleSelection)
            {
                foreach (BarSeries3D series in this.chart.Series)
                {
                    foreach (XyzDataPoint3D dp in series.DataPoints)
                    {
                        var currentInfo = (PlotInfo)dp.DataItem;
                        if (currentInfo != plotInfo)
                        {
                            currentInfo.IsSelected = false;
                        }
                    }
                }
            }
            plotInfo.IsSelected ^= true;
        }

        private Visual3D HitTest(Viewport3D viewport, Point position)
        {
            Visual3D visualHit = null;

            HitTestResultCallback callback = result =>
            {
                RayMeshGeometry3DHitTestResult rayMeshResult = result as RayMeshGeometry3DHitTestResult;

                visualHit = rayMeshResult.VisualHit;

                return HitTestResultBehavior.Stop;
            };
            VisualTreeHelper.HitTest(viewport, null, callback, new PointHitTestParameters(position));

            return visualHit;
        }

        private RadRect3D GetLayoutSlot(Visual3D visualHit)
        {
            var propertyInfo = visualHit.GetType().GetProperty("LayoutSlot", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
            var layoutSlot = (Rect3D)propertyInfo.GetValue(visualHit);
            var radLayoutSlot = new RadRect3D(new RadPoint3D(layoutSlot.X, layoutSlot.Y, layoutSlot.Z), new RadSize3D(layoutSlot.SizeX, layoutSlot.SizeY, layoutSlot.SizeZ));
            return radLayoutSlot;
        }

        private void ResetMaterialSelector(BarSeries3D series)
        {
            var materialSelector = series.DefaultVisualMaterialSelector;
            series.DefaultVisualMaterialSelector = null;
            series.DefaultVisualMaterialSelector = materialSelector;
        }
    }

    public class PlotInfo : ViewModelBase
    {
        public string XValue { get; set; }
        public string YValue { get; set; }
        public double ZValue { get; set; }
        public bool IsSelected { get; set; }
    }

    public class DefaultMaterialSelector : MaterialSelector
    {
        public Material NormalMaterial { get; set; }
        public Material SelectedMaterial { get; set; }

        public override Material SelectMaterial(object context)
        {
            XyzDataPoint3D dp = (XyzDataPoint3D)context;
            PlotInfo info = (PlotInfo)dp.DataItem;
            if (info.IsSelected)
            {
                return this.SelectedMaterial;
            }
            return this.NormalMaterial;
        }
    }
}
